﻿//////////////////////////////////////////////
// GlesRenderer.h
//
// This renderer focuses on using ES available API for GL work.
// The aim is to work on desktop but also Android
// Another point is to be able to be translated to WebGL by Emscripten
//
// As such, it should only play with ES2 / ES3 available functions
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics::gles
{
	class GlesDataAllocator ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../../Dll/DllDefines.h"

#include "../Renderer.h"

/// Class ------------------------------------

namespace nkGraphics::gles
{
	class DLL_GRAPHICS_EXPORT GlesRenderer final : public nkGraphics::Renderer
	{
		private :

			// Attributes
			// Allocator for data
			GlesDataAllocator* _allocator ;

		public :
	
			// Constructor, destructor
			GlesRenderer () ;
			~GlesRenderer () ;

			// Getters

			// Setters

			// Overrides
			// Lifetime
			virtual bool initialize (const ConfigHolder& config) override ;
			virtual bool initiateBuiltInShaders () override ;
			virtual bool postGraphicSystemInit () override ;
			virtual void prepareForRelease () override ;
			virtual bool shutdown () override ;

			// Flush
			virtual void flushRendering () override ;

			// Rendering
			virtual void preRenderRoutine () override ;
			virtual void postRenderRoutine () override ;
			virtual bool launchComputeShader (Shader* shader, unsigned int xThread, unsigned int yThread, unsigned int zThread) override ;
			virtual bool launchGeometryShaderStream (Shader* shader, Mesh* baseMesh, Mesh* outputMesh, Buffer* outputBuffer, bool gpuCopy = false, bool async = false) override ;

			// External getters
			virtual DataAllocator* getDataAllocator () const override ;
			virtual RendererSupportInfo getRendererSupportInfo () const override ;
	} ;
}