﻿//////////////////////////////////////////////
// GlesShaderDataFeeder.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class QueuedRenderableObject ;
}

namespace nkGraphics::gles
{
	class GlesBuffer ;
	class GlesCommandList ;
	class GlesShader ;
}

/// Includes ---------------------------------

// Standards
#include <vector>

/// Class ------------------------------------
	
namespace nkGraphics::gles
{
	class GlesShaderDataFeeder final
	{
		private :
		
			// Attributs
			// Instance buffers
			std::vector<GlesBuffer*> _instanceBuffers ;
			// CPU data to fill the buffers
			std::vector<unsigned char*> _instanceCpuMemory ;

			// Parent shader
			GlesShader* _parentShader ;

			// Counters
			unsigned int _bufferCount ;
			unsigned int _instanceCounter ;
			unsigned int _instanceDataSize ;

		private :

			// Functions
			// Data commit
			void _commitInstanceData () ;
		
		public :
		
			// Constructor, destructor
			GlesShaderDataFeeder () ;
			~GlesShaderDataFeeder () ;

			// Setters
			void setParentShader (GlesShader* value) ;

			// Functions shaders
			void prepareForComputePassUse (GlesCommandList* cBuffer) ;
			void prepareForPostProcessPassUse (GlesCommandList* cBuffer) ;
			void prepareForScenePassUse (GlesCommandList* cBuffer) ;

			bool prepareForRenderable (QueuedRenderableObject* renderable, GlesCommandList* cBuffer) ;

			void noticeComputeShaderActivated (GlesShader* lastUsedShader, GlesCommandList* cBuffer) ;
			void noticeGraphicsShaderActivated (GlesShader* lastUsedShader, GlesCommandList* cBuffer) ;

			void preCommandBufferExecution () ;
			void postComputeCommandBufferExecution () ;
			void postGraphicsCommandBufferExecution () ;

			// Functions pratiques
			void checkInstanceData () ;
			void checkInstanceBuffer (GlesCommandList* commandList) ;
			bool createInstanceBuffer () ;
	} ;
}